package ru.pricepulse.parsingservice.wildberries_parser.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.transaction.Transactional;
import lombok.AllArgsConstructor;
import org.springframework.core.convert.ConversionService;
import org.springframework.stereotype.Service;
import ru.pricepulse.parsingservice.persistence.entity.PriceHistoryEntity;
import ru.pricepulse.parsingservice.persistence.entity.PriceHistoryId;
import ru.pricepulse.parsingservice.persistence.entity.ProductEntity;
import ru.pricepulse.parsingservice.persistence.repository.ProductPriceRepository;
import ru.pricepulse.parsingservice.persistence.repository.ProductRepository;
import ru.pricepulse.parsingservice.wildberries_parser.configuration.WbProperties;
import ru.pricepulse.parsingservice.wildberries_parser.service.client.Client;
import ru.pricepulse.parsingservice.wildberries_parser.service.dto.ProductInfoDto;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@AllArgsConstructor
public class WildberriesParsingService {
    private final Client client;
    private final ObjectMapper objectMapper;
    private final ConversionService conversionService;
    private final ProductRepository productRepository;
    private final WbProperties wbProperties;
    private final ProductPriceRepository productPriceRepository;

    @Transactional
    public void parse() {
        List<ProductEntity> productEntities = new ArrayList<>();
        List<PriceHistoryEntity> priceHistories = new ArrayList<>();

        final int elementsInPage = 100;
        int page = 1;
        Integer totalPages = null;

        do {
            var pageData = client.scrapPage(page, wbProperties.getShard(), wbProperties.getLaptopUrl());

            if (totalPages == null) {
                Map<String, Object> dataMap = (Map<String, Object>) pageData.get("data");
                int totalElements = (int) dataMap.get("total");
                totalPages = (int) Math.ceil((double) totalElements / elementsInPage);
            }

            List<ProductInfoDto> productInfoDtoList = convertMapObjectToListProductInfoDto(pageData);

            productInfoDtoList.forEach(dto -> {

                ProductEntity productEntity = conversionService.convert(dto, ProductEntity.class);

                PriceHistoryEntity priceHistory = PriceHistoryEntity.builder()
                        .id(new PriceHistoryId(productEntity.getUrl(), LocalDateTime.now()))
                        .price(BigDecimal.valueOf(dto.getSalePriceU() / 100.0))
                        .build();

                productEntities.add(productEntity);
                priceHistories.add(priceHistory);
            });

            page++;
        } while (page <= totalPages);

        productRepository.saveAll(productEntities);
        productPriceRepository.saveAll(priceHistories);
    }

    private List<ProductInfoDto> convertMapObjectToListProductInfoDto(Map<String, Object> map) {
        Map<String, ArrayList<Object>> dataMap = (Map<String, ArrayList<Object>>) map.get("data");
        return getProductInfoDtos(dataMap);
    }

    private List<ProductInfoDto> getProductInfoDtos(Map<String, ArrayList<Object>> dataMap) {
        return objectMapper.convertValue(
                dataMap.get("products"),
                new TypeReference<>() {}
        );
    }
}
